<?php
/**
 * The virtual path to use for VFS data.
 */
define('AGORA_VFS_PATH', '.horde/agora/attachments/');

/**
 * The Agora:: class provides basic Agora functionality.
 *
 * Copyright 2003-2005 Marko Djukic <marko@oblo.com>
 *
 * See the enclosed file COPYING for license information (GPL). If you did not
 * receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * $Horde: agora/lib/Agora.php,v 1.75 2005/01/03 14:34:46 jan Exp $
 *
 * @author Marko Djukic <marko@oblo.com>
 * @version $Revision: 1.75 $
 * @package Agora
 */
class Agora {

    /**
     * Determines the requested forum_id, thread_id and message_id by
     * checking first if they are passed as the single encoded var or
     * individual vars.
     *
     * @access public
     *
     * @return array  Forum, thread and message id.
     */
    function getAgoraId()
    {
        if (($id = Util::getFormData('agora')) !== null) {
            if (strstr($id, '.')) {
                list($forum_id, $message_id) = explode('.', $id, 2);
            } else {
                $forum_id = $id;
                $message_id = 0;
            }
        } else {
            $forum_id = Util::getFormData('forum_id');
            $message_id = Util::getFormData('message_id');
        }

        return array($forum_id, $message_id);
    }

    /**
     * Creates the Agora id.
     *
     * @access public
     *
     * @return string  If passed with the $url parameter, returns a completed
     *                 url with the agora_id tacked on at the end, otherwise
     *                 returns the simple agora_id.
     */
    function setAgoraId($forum_id, $message_id, $url = '')
    {
        $agora_id = $forum_id . '.' . $message_id;

        if (!empty($url)) {
            return Util::addParameter($url, 'agora', $agora_id);
        }

        return $agora_id;
    }

    /**
     * Function that works out the forum ID from a given DataTreen name for a
     * message.
     *
     * @access public
     *
     * @param string $category_name  A ':' separated DataTree name.
     *
     * @return int
     */
    function getForumId($category_name)
    {
        list($forum_id) = explode(':', $category_name);
        return $forum_id;
    }

    /**
     * Formats a list of forums, showing each child of a parent with
     * appropriate indent using '.. ' as a leader.
     *
     * @access public
     *
     * @param array $forums  The list of forums to format.
     *
     * @return array  Formatted forum list.
     */
    function formatCategoryTree($forums)
    {
        foreach ($forums as $id => $forum) {
            $levels = explode(':', $forum);
            $forums[$id] = str_repeat('.. ', count($levels) - 1) . array_pop($levels);
        }
        return $forums;
    }

    /**
     * Returns the column to sort by, checking first if it is specified in the
     * URL, then returning the value stored in prefs.
     *
     * @access public
     *
     * @param string $view  The view name, used to identify preference settings
     *                      for sorting.
     *
     * @return string  The column to sort by.
     */
    function getSortBy($view)
    {
        global $prefs;
        if (($sortby = Util::getFormData($view . '_sortby')) !== null) {
            $prefs->setValue($view . '_sortby', $sortby);
        }
        return $prefs->getValue($view . '_sortby');
    }

    /**
     * Returns the sort direction, checking first if it is specified in the URL,
     * then returning the value stored in prefs.
     *
     * @access public
     *
     * @param string $view  The view name, used to identify preference settings
     *                      for sorting.
     *
     * @return integer  The sort direction, 0 = ascending, 1 = descending.
     */
    function getSortDir($view)
    {
        global $prefs;
        if (($sortdir = Util::getFormData($view . '_sortdir')) !== null) {
            $prefs->setValue($view . '_sortdir', $sortdir);
        }
        return $prefs->getValue($view . '_sortdir');
    }

    /**
     * Formats column headers have sort links and sort arrows.
     *
     * @access public
     *
     * @param array  $columns   The columns to format.
     * @param string $sort_by   The current 'sort-by' column.
     * @param string $sort_dir  The current sort direction.
     * @param string $view      The view name, used to identify preference
     *                          settings for sorting.
     *
     * @return array  The formated column headers to be displayed.
     */
    function formatColumnHeaders($columns, $sort_by, $sort_dir, $view)
    {
        /* Get the current url, remove any sorting parameters. */
        $url = Horde::selfURL(true);
        $url = Util::removeParameter($url, array($view . '_sortby', $view . '_sortdir'));

        /* Go through the column headers to format and add sorting links. */
        $headers = array();
        foreach ($columns as $col_name => $col_title) {
            $extra = array();
            /* Is this a column with two headers? */
            if (is_array($col_title)) {
                $keys = array_keys($col_title);
                $extra_name = $keys[0];
                if ($sort_by == $keys[1]) {
                    $extra = array($keys[0] => $col_title[$keys[0]]);
                    $col_name = $keys[1];
                    $col_title = $col_title[$keys[1]];
                } else {
                    $extra = array($keys[1] => $col_title[$keys[1]]);
                    $col_name = $keys[0];
                    $col_title = $col_title[$keys[0]];
                }
            }
            if ($sort_by == $col_name) {
                /* This column is currently sorted by, plain title and
                 * add sort direction arrow. */
                $sort_img = ($sort_dir ? 'za.png' : 'az.png');
                $sort_title = ($sort_dir ? _("Sort Ascending") : _("Sort Descending"));
                $col_arrow = Horde::link(Util::addParameter($url, array($view . '_sortby' => $col_name, $view . '_sortdir' => $sort_dir ? 0 : 1)), $sort_title) .
                    Horde::img($sort_img, $sort_title, null, $GLOBALS['registry']->getImageDir('horde')) . '</a>';
                $col_class = 'selected';
            } else {
                /* Column not currently sorted, add link to sort by
                 * this one and no sort arrow. */
                $col_arrow = '';
                $col_title = Horde::link(Util::addParameter($url, $view . '_sortby', $col_name), sprintf(_("Sort by %s"), $col_title)) . $col_title . '</a>';
                $col_class = '';
            }
            if (count($extra)) {
                list($name, $title) = each($extra);
                $col_title .= '&nbsp;<span style="font-size: 10px;">[';
                $col_title .= Horde::link(Util::addParameter($url, $view . '_sortby', $name), sprintf(_("Sort by %s"), $title), '" style="font-size: 10px; font-weight: normal;') . $title . '</a>';
                $col_title .= ']</span>';
                $col_name = $extra_name;
            }
            $headers[$col_name] = $col_arrow . $col_title;
            $headers[$col_name . '_class_plain'] = $col_class;
            $headers[$col_name . '_class'] = empty($col_class) ? '' : ' class="' . $col_class . '"';
        }

        return $headers;
    }

    /**
     * Returns a {@link VFS} instance.
     *
     * @return VFS  A VFS instance.
     */
    function &getVFS()
    {
        global $conf;

        if (!isset($conf['vfs']['type'])) {
            return PEAR::raiseError(_("The VFS backend needs to be configured to enable attachment uploads."), 'horde.error');
        }

        require_once 'VFS.php';
        return VFS::singleton($conf['vfs']['type'], Horde::getDriverConfig('vfs'));
    }

    function getMenu($returnType = 'object')
    {
        require_once 'Horde/Menu.php';

        $menu = &new Menu();

        /* Agora Home. */
        $menu->add(Horde::applicationUrl('forums.php'), _("Forums"), 'forums.png', $GLOBALS['registry']->getImageDir(), null, null,
                   dirname($_SERVER['PHP_SELF']) == $GLOBALS['registry']->get('webroot') && basename($_SERVER['PHP_SELF']) == 'index.php' ? 'current' : null);

        /* Thread list, if applicable. */
        if (isset($GLOBALS['forum_id'])) {
            $menu->add(Agora::setAgoraId($GLOBALS['forum_id'], null, Horde::applicationUrl('threads.php')), _("Threads"), 'threads.png', $GLOBALS['registry']->getImageDir());
            if (Auth::getAuth()) {
                $menu->add(Agora::setAgoraId($GLOBALS['forum_id'], null, Horde::applicationUrl('messages/edit.php')), _("New Thread"), 'newmessage.png', $GLOBALS['registry']->getImageDir());
            }
        }

        if (Auth::isAdmin('agora:admin')) {
            $menu->add(Horde::applicationUrl('editforum.php'), _("New Forum"), 'newforum.png', $GLOBALS['registry']->getImageDir());
            $menu->add(Horde::applicationUrl('moderate.php'), _("Moderate"), 'moderate.png', $GLOBALS['registry']->getImageDir());
        }

        if ($returnType == 'object') {
            return $menu;
        } else {
            return $menu->render();
        }
    }

}
