<?php

require_once AGORA_BASE . '/lib/Agora.php';

/**
 * Agora_Forums:: class provides the functions to access forums.
 *
 * $Horde: agora/lib/Forums.php,v 1.61 2005/01/03 14:34:46 jan Exp $
 *
 * Copyright 2003-2005 Marko Djukic <marko@oblo.com>
 *
 * See the enclosed file COPYING for license information (GPL). If you did not
 * receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @author  Marko Djukic <marko@oblo.com>
 * @version $Revision: 1.61 $
 * @package Agora
 */
class Agora_Forums {

    /**
     * Pointer to a category instance to manage the forums and
     * messages.
     *
     * @var object Category $_forums
     */
    var $_forums;

    /**
     * Constructor.
     *
     * @param string $data_root  The root DataTree group to use.
     *                           Defaults to 'agora.forums'.
     */
    function Agora_Forums($data_root = 'agora.forums')
    {
        global $conf;

        if (empty($conf['datatree']['driver'])) {
            Horde::fatal(_("You must configure a Horde Categories backend to use Agora."), __FILE__, __LINE__);
        }
        $driver = $conf['datatree']['driver'];
        $params = array_merge(Horde::getDriverConfig('datatree', $driver), array('group' => $data_root));

        /* Connect to the backend for tracking forums. */
        $this->_forums = &DataTree::singleton($driver, $params);
    }

    /**
     * Attempts to return a reference to a concrete Forums instance. It will
     * only create a new instance if no Forums instance currently exists.
     *
     * This method must be invoked as: $var = &Agora_Forums::singleton();
     *
     * @param string $data_root  The root DataTree group to use.
     *                           Defaults to 'agora.forums'.
     *
     * @return object Forums The concrete Forums reference, or false
     *                        on an error.
     */
    function &singleton($data_root = 'agora.forums')
    {
        static $forums = array();

        if (!isset($forums[$data_root])) {
            $forums[$data_root] = new Agora_Forums($data_root);
        }

        return $forums[$data_root];
    }

    function &newForum($forum_name)
    {
        if (empty($forum_name)) {
            return PEAR::raiseError(_("Can not create a forum with an empty name."));
        }
        $forum = &new Forum_DataTreeObject($forum_name);
        $forum->setForumsOb($this);
        return $forum;
    }

    /**
     * Returns whether a forum exists or not.
     *
     * @param string $forum_name  A forum name or object.
     *                            @see DataTree::exists.
     *
     * @return boolean  True if the specified forum exists.
     */
    function exists($forum_name)
    {
        return $this->_forums->exists($forum_name);
    }

    /**
     * Fetches a forum DataTree object.
     *
     * @access public
     *
     * @param integer $forum_id  The ID of the forum to fetch.
     *
     * @return object  The forum object or a PEAR_Error on failure.
     */
    function &getForum($forum_id)
    {
        $forum = &$this->_forums->getObjectById($forum_id, 'Forum_DataTreeObject');
        /* Found a valid forum, set the forum object. */
        if (!is_a($forum, 'PEAR_Error')) {
            $forum->setForumsOb($this);
        }

        return $forum;
    }

    /**
     * Returns an ID for a given forum name.
     *
     * @access public
     *
     * @param string $forum  The full forum name, inclusive of any parents and
     *                       separated by ':'.
     *
     * @return integer  The ID of the forum.
     */
    function getForumId($forum)
    {
        return $this->_forums->getId($forum);
    }

    /**
     * Returns the ID of the parent for a given forum.
     *
     * @access public
     *
     * @param object $forum  The forum DataTree object for which to return the
     *                       parent ID.
     *
     * @return integer  The ID of the parent forum.
     */
    function getForumParent($forum)
    {
        return $this->_forums->getParent($forum);
    }

    function getForumPath($id)
    {
        return $this->_forums->getName($id);
    }

    /**
     * Fetches a list of forums.
     *
     * @access public
     *
     * @param integer  $root_forum  The first level forum.
     * @param boolean  $formatted   Whether to return the list formatted or raw.
     * @param string   $sort_by     The column to sort by.
     * @param integer  $sort_dir    Sort direction, 0 = ascending, 1 = descending.
     * @param boolean  $no_root     Remove the root forum.
     *
     * @return mixed  An array of forums or PEAR_Error on failure.
     */
    function getForums($root_forum = DATATREE_ROOT, $formatted = true, $sort_by = 'forum_name', $sort_dir = 0, $no_root = false)
    {
        global $registry;

        /* Get root forum name and calculate base indent. */
        $root_forum_name = $this->_forums->getName($root_forum);
        $base_indent = substr_count($root_forum_name, ':');

        $forums_tree = $this->_forums->get(DATATREE_FORMAT_FLAT, $root_forum_name, true, _("-- Top Level --"));

        /* If root is not required, unset it out of the array. */
        if ($no_root) {
            unset($forums_tree[$root_forum]);
            /* If there is an actual base indent, shorten it by one. */
            if ($base_indent > 0) {
                $base_indent--;
            }
        }

        if (!$formatted) {
            return $forums_tree;
        }

        if (count($forums_tree)) {
            require_once AGORA_BASE . '/lib/Messages.php';
            require_once 'Horde/Array.php';
        }

        $forums = &$this->_forums->getObjects(array_keys($forums_tree), 'Forum_DataTreeObject');

        $forums_list = array();
        foreach ($forums_tree as $forum_id => $forum_name) {
            if ($sort_by != 'flat_forum_name') {
                /* Calculate indent level. */
                $forums_list[$forum_id]['indentn'] = substr_count($forum_name, ':') - $base_indent;
                $forums_list[$forum_id]['indent'] = str_repeat('.. ', $forums_list[$forum_id]['indentn']);
            } else {
                /* No indent on flat sort by forum name. */
                $forums_list[$forum_id]['indentn'] =  0;
                $forums_list[$forum_id]['indent'] = '';
            }

            $forums_list[$forum_id]['url'] = Agora::setAgoraId($forum_id, null, Horde::applicationUrl('threads.php'));
            $forums_list[$forum_id]['forum_name'] = $forum_name;
            $forums_list[$forum_id]['flat_forum_name'] = DataTree::getShortName($forum_name);

            /* Set the forum description. */
            $forums_list[$forum_id]['forum_desc'] = $forums[$forum_name]->get('forum_description');

            /* Get the number of messages. */
            $threads = &Agora_Messages::singleton(array('forum_id' => $forum_id));
            $messages = $threads->getThreads($forum_id, true);
            $forums_list[$forum_id]['message_count'] = count($messages);
            if ($forums_list[$forum_id]['message_count'] > 0) {
                Horde_Array::arraySort($messages, 'message_timestamp', 0);
                $last = array_pop($messages);
                $forums_list[$forum_id]['last_message_subject'] = $last['message_subject'];
                $forums_list[$forum_id]['last_message_author'] = $last['message_author'];
                $forums_list[$forum_id]['last_message_date'] = strftime($GLOBALS['prefs']->getValue('date_format'), $last['message_timestamp']);
                $forums_list[$forum_id]['last_message_url'] = Agora::setAgoraId($forum_id, $last['message_id'], Horde::applicationUrl('messages/index.php'));
            } else {
                $forums_list[$forum_id]['last_message_subject'] = null;
                $forums_list[$forum_id]['last_message_author'] = null;
                $forums_list[$forum_id]['last_message_date'] = null;
                $forums_list[$forum_id]['last_message_url'] = null;
            }

            /* Action links. */
            $forums_list[$forum_id]['actions'] = array();

            /* Post message button. */
            $url = Agora::setAgoraId($forum_id, null, Horde::applicationUrl('messages/edit.php'));
            $forums_list[$forum_id]['actions'][] = Horde::link($url, _("Post message"), '', '', '', _("Post message")) . Horde::img('newmessage.png', _("Post message")) . '</a>';

            if (Auth::isAdmin('agora:admin')) {
                /* Edit forum button. */
                $url = Agora::setAgoraId($forum_id, null, Horde::applicationUrl('editforum.php'));
                $forums_list[$forum_id]['actions'][] = Horde::link($url, _("Edit forum"), '', '', '', _("Edit forum")) . Horde::img('edit.png', _("Edit Forum"), '', $registry->getImageDir('horde')) . '</a>';

                /* Delete forum button. */
                $url = Agora::setAgoraId($forum_id, null, Horde::applicationUrl('deleteforum.php'));
                $forums_list[$forum_id]['actions'][] = Horde::link($url, _("Delete forum"), '', '', '', _("Delete forum")) . Horde::img('delete.png', _("Delete forum"), null, $registry->getImageDir('horde')) . '</a>';
            }
        }

        /* Sort the forum list. */
        require_once 'Horde/Array.php';
        Horde_Array::arraySort($forums_list, $sort_by, $sort_dir);
        return $forums_list;
    }

    /**
     * Fetches a list of forums.
     *
     * @access public
     *
     * @param integer $root_forum  The first level forum.
     *
     * @return integer  The number of forums or PEAR_Error on failure.
     */
    function countForums($root_forum = DATATREE_ROOT)
    {
        return $this->_forums->count($root_forum);
    }

    /**
     * Saves a forum, either creating one if no forum ID is given or updating
     * an existing one.
     *
     * @param array $info  The forum information to save consisting of:
     *                       forum_id
     *                       forum_parent_id
     *                       forum_name
     *                       forum_moderated
     *                       forum_description
     *                       forum_attachments
     *
     * @return mixed  The forum ID on success or PEAR_Error on failure.
     */
    function saveForum(&$info)
    {
        if (empty($info['forum_id'])) {
            /* If root, path is blank. */
            $forum_path = ($info['forum_parent_id'] != DATATREE_ROOT ? $this->getForumPath($info['forum_parent_id']) . ':' : '');
            $forum_path .= $info['forum_name'];
            $forum = &$this->newForum($forum_path);
            $forum->data['message_seq'] = 0;
        } else {
            $forum = $this->getForum($info['forum_id']);
        }

        /* Set the forum data fields. */
        $forum->data['forum_moderated'] = $info['forum_moderated'];
        $forum->data['forum_description'] = $info['forum_description'];
        if (isset($info['forum_attachments'])) {
            $forum->data['forum_attachments'] = $info['forum_attachments'];
        }

        if (empty($info['forum_id'])) {
            /* New forum, add the category object. */
            $result = $this->_forums->add($forum);
            return $this->_forums->getId($forum);
        } else {
            /* Existing forum, first save the category data. */
            $this->_forums->updateData($forum);
            $old_parent_id = $this->getForumParent($forum);
            $old_forum_cat = $forum->getName();
            /* Check if the old parent id is still the same. */
            if ($old_parent_id != $info['forum_parent_id']) {
                /* Get the new parent path. */
                $new_parent = $this->_forums->getName($info['forum_parent_id']);
                /* Move the forum. */
                $move = $this->_forums->move($old_forum_cat, $new_parent);
                if (is_a($move, 'PEAR_Error')) {
                    return $move;
                }
                $old_forum_cat = $new_parent . ':' . $forum->getShortName();
            }
            /* If the forum name has changed, rename the category. */
            if ($forum->getShortName() != $info['forum_name']) {
                $rename = $this->_forums->rename($old_forum_cat, $info['forum_name']);
                if (is_a($rename, 'PEAR_Error')) {
                    return $rename;
                }
            }
            return $info['forum_id'];
        }
    }

    /**
     * Delets a forum, any subforums that are present and all messages
     * contained in the forum and subforums.
     *
     * @param integer $forum_id  The ID of the forum to delete.
     *
     * @return mixed  True on success or PEAR_Error on failure.
     */
    function deleteForum($forum_id)
    {
        /* Delete first all the threads in the forum. */
        require_once AGORA_BASE . '/lib/Messages.php';
        $params = array('forum_id' => $forum_id);
        $messages = &Agora_Messages::singleton($params);
        $result = $messages->deleteThread();
        if (is_a($result, 'PEAR_Error')) {
            return $result;
        }

        /* Delete the forum itself. */
        $forum = $this->getForum($forum_id);
        return $this->_forums->remove($forum, true);
    }

    function getPathByName($id)
    {
        return $this->_forums->_nameMap[$id];
    }

    function getMessageReplyCount($message_id = '')
    {
        if (empty($message_id)) {
            return 0;
        }
        return $this->_messages->getNumberOfChildren($this->_messages->getName($message_id));
    }

    function getNumberOfSubjects($forum_id)
    {
        return count($this->getSubjectsList($forum_id));
    }

}

class Forum_DataTreeObject extends DataTreeObject {

    var $_forumsOb;

    function Forum_DataTreeObject($name)
    {
        parent::DataTreeObject($name);
    }

    function setForumsOb(&$forumsOb)
    {
        $this->_forumsOb = &$forumsOb;
    }

    function getForumArray()
    {
        return array('forum_id' => $this->_forumsOb->getForumId($this->getName()),
                     'forum_parent_id'   => $this->_forumsOb->getForumParent($this),
                     'forum_name'        => $this->getShortName(),
                     'forum_moderated'   => $this->get('forum_moderated'),
                     'forum_description' => $this->get('forum_description'),
                     'forum_attachments' => $this->get('forum_attachments'));
    }

    function saveForumData()
    {
        return $this->_forumsOb->_forums->updateData($this);
    }

    /**
     * Check if this is a moderated forum.
     */
    function isModerated()
    {
        return $this->data['forum_moderated'];
    }

    function _toAttributes()
    {
        $attributes[] = array('name'  => 'message',
                              'key'   => 'seq',
                              'value' => $this->data['message_seq']);
        $attributes[] = array('name'  => 'forum',
                              'key'   => 'moderated',
                              'value' => !empty($this->data['forum_moderated']) ? 1 : 0);
        $attributes[] = array('name'  => 'forum',
                              'key'   => 'description',
                              'value' => $this->data['forum_description']);
        $attributes[] = array('name'  => 'forum',
                              'key'   => 'attachments',
                              'value' => !empty($this->data['forum_attachments']) ? 1 : 0);
        return $attributes;
    }

    function _fromAttributes($attributes)
    {
        /* Initialise the data array. */
        $this->data = array();

        foreach ($attributes as $attr) {
            $this->data[$attr['name'] . '_' . $attr['key']] = $attr['value'];
        }
    }

}
